@@ -16,7 +16,13 @@ def solve(formula, display=True, params={}):
 
     try:
         if formula.qmat:
-            warnings.warn('SOC constriants are ignored in the LP solver. ')
+            warnings.warn('the LP solver ignnores SOC constriants.')
+    except AttributeError:
+        pass
+
+    try:
+        if formula.xmat:
+            warnings.warn('The LP solver ignores exponential cone constriants.')
     except AttributeError:
         pass
 
